/**
	$Id: PlotGenerator.java 5218 2010-01-18 07:31:09Z jari $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.extensions.plot;

/**
	Contains information about a single image in an overview plot.
	The information returned by implementations are usually placed
	as properties in &lt;img&gt; tags.

	@author Nicklas
	@version 2.12
	@base.modified $Date: 2010-01-18 08:31:09 +0100 (må, 18 jan 2010) $
*/
public interface PlotGenerator
{
	/**
		Get the ID of the image. It must be unique among the
		images generated for a single action. This ID is combined with
		the ID of the action and extension to generate an application-wide
		unique ID.
	*/
	public String getId();
	
	/**
		Get the URL to a servlet that is used to generate the image.
		This information goes into the "src" attribute.
	*/
	public String getUrl();
	
	/**
		Get the width of the generated image.
	*/
	public int getWidth();
	
	/**
		Get the height of the generated image.
	*/
	public int getHeight();
	
	/**
		Get the title of the generated image. This information
		goes into the "title" attribute.
	*/
	public String getTitle();
}
