/**
	$Id: ServiceControllerAction.java 4512 2008-09-11 20:05:50Z jari $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.extensions.service;

import net.sf.basedb.util.extensions.Action;

/**
	Action for controlling services installed under the extensions system.
	A factory may return the same or a different action on each invokation,
	but the action must control the same service. The actions should not store
	state internally. Eg. if there are two different controllers for the same
	extension the {@link #isRunning()} method should always return the same
	value on both of them and invoking {@link #start()} or {@link #stop()} on one
	controller should be immediately visible on the second controller.

	@author nicklas
	@version 2.8
	@base.modified $Date: 2008-09-11 22:05:50 +0200 (to, 11 sep 2008) $
*/
public interface ServiceControllerAction
	extends Action
{

	/**
		Is the service running or not?
	*/
	public boolean isRunning();
	
	/**
		Start the service if it is not already running. If this method
		is called on a running service it should do nothing. If the service can't
		be started it may throw an exception.
	*/
	public void start();
	
	/**
		Stop the service if it is running. If this method is called on a 
		service that is not running, it should do nothing. If the service
		can't be stopped it may throw an exception.
	*/
	public void stop();
	
}
