/**
	$Id: ServicesEventHandler.java 6875 2015-04-20 09:08:18Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.extensions.service;

import net.sf.basedb.clients.web.extensions.ExtensionsControl;
import net.sf.basedb.core.Application;
import net.sf.basedb.util.extensions.Extension;
import net.sf.basedb.util.extensions.ExtensionPoint;
import net.sf.basedb.util.extensions.events.EventHandler;
import net.sf.basedb.util.extensions.events.EventType;
import net.sf.basedb.util.extensions.manager.Settings;

/**
	Event handler for managing services when new extension are added
	or existing extension are updated or deleted.

	@author nicklas
	@version 2.8
	@base.modified $Date: 2015-04-20 11:08:18 +0200 (må, 20 apr 2015) $
*/
public class ServicesEventHandler
	implements EventHandler
{

	private static final org.slf4j.Logger log = 
		org.slf4j.LoggerFactory.getLogger(ServicesEventHandler.class);

	private final Settings settings;
	
	public ServicesEventHandler()
	{
		this.settings = Application.getExtensionsManager().getSettings();
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public void handleEvent(EventType event, ExtensionPoint<?> extensionPoint, Extension<?> extension)
	{
		if (log.isInfoEnabled())
		{
			log.info("Received event: " + event + "; extension point: " + extensionPoint + 
					"; extension: " + extension);
		}
		if (event == EventType.AFTER_REGISTRATION)
		{
			// Start services
			if (extension == null || settings.isEnabled(extension))
			{
				Services.start((Extension<ServiceControllerAction>)extension);
			}
		}
		else if (event == EventType.BEFORE_UPDATE)
		{
			// Stop running services from this extension
			Services.stop((Extension<ServiceControllerAction>)extension);
		}
		else if (event == EventType.AFTER_UPDATE)
		{
			// Re-start services that were running
			if (extension == null || settings.isEnabled(extension))
			{
				Services.start((Extension<ServiceControllerAction>)extension);
			}
		}
		else if (event == EventType.BEFORE_UNREGISTRATION)
		{
			// Stop running services
			Services.stop((Extension<ServiceControllerAction>)extension);
		}
		else if (event == ExtensionsControl.BEFORE_DISABLE)
		{
			// Stop running services
			Services.stop((Extension<ServiceControllerAction>)extension);
		}
		else if (event == ExtensionsControl.AFTER_ENABLE)
		{
			// Start services
			Services.start((Extension<ServiceControllerAction>)extension);
		}
		else
		{
			log.info("No handler for event " + event);
		}
	}

}
