/*
	$Id: FileStoreEnabled.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Collection;

/**
	This interface is implemented by items that can use files instead
	of the database for storing data. For example, to store raw data
	for a {@link RawBioAssay} and features for a {@link ArrayDesign}.

	@author Nicklas
	@version 2.5
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (to, 11 sep 2008) $
*/
public interface FileStoreEnabled
	extends Identifiable
{

	/**
		Get the file set that collects all files that holds data
		for this item. If no file set exists a new one is created
		and automatically saved to the database when {@link DbControl#commit()}
		is called. To check if an item has files without creating a new 
		file set use the {@link #hasFileSet()} method.
		@return A file set
	*/
	public FileSet getFileSet();
	
	/**
		Check if this item has a file set. Always call this method before
		{@link #getFileSet()} to avoid creating a new file set when there is 
		no need for it.
		@return TRUE if a file set exists, FALSE if not
	*/
	public boolean hasFileSet();
	
	/**
		Get the platform the item is related to. The platform is used to
		limit the {@link DataFileType}:s that can be added to the file set.
		@return A platform or null if the items is not related to any platform.
	*/
	public Platform getPlatform();
	
	/**
		Get the platform variant the item is related to. The platform is used to
		limit the {@link DataFileType}:s that can be added to the file set.
		@return A platform or null if the items is not related to any variant.
	*/
	public PlatformVariant getVariant();

	/**
		Get the file sets of parent item(s) that are related to the current item.
		If the item has no parent(s) or if the parent has no file set, null or
		an empty collection is returned.
		@return A collection of <code>FileSet</code> or null
	*/
	public Collection<FileSet> getParentFileSets();
}
