/**
	$Id$

	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.node;

import net.sf.basedb.core.BioWell;
import net.sf.basedb.core.MeasuredBioMaterial;
import net.sf.basedb.util.overview.Node;

/**
	Name generator implementation for biomaterial items
	that are located on a biowell. This name generator is used when
	loading biomaterial on a bioplate. 
	
	@author Nicklas
	@since 3.2
	@base.modified $Date$
*/
public class BioWellNameGenerator
	implements NodeNameGenerator<MeasuredBioMaterial>
{

	private BioWell currentWell;
	
	/**
		Create a new biowell name generator.
	*/
	public BioWellNameGenerator()
	{}
	
	/*
		From the NodeNameGenerator interface
		------------------------------------
	*/
	@Override
	public String getNodeTitle(MeasuredBioMaterial item, Node parentNode)
	{
		return getCurrentTitlePrefix() + item.getName();
	}

	@Override
	public String getNodeName(MeasuredBioMaterial item, Node parentNode) 
	{
		return getCurrentNodeName();
	}

	@Override
	public String getMissingNodeTitle(Node parentNode) 
	{
		// Should never happen since the loader is skipping empty wells
		// but in just in case...
		return getCurrentTitlePrefix() + ": missing";
	}

	@Override
	public String getMissingNodeName(Node parentNode) 
	{
		return getCurrentNodeName();
	}

	@Override
	public String getDeniedNodeTitle(Node parentNode) 
	{
		return getCurrentTitlePrefix() + ": denied";
	}

	@Override
	public String getDeniedNodeName(Node parentNode) 
	{
		return getCurrentNodeName();
	}


	// -------------------------------------
	
	/**
		Set the well we are currently working with. The name generator need to
		know this before trying to generate node names and titles since it is
		possible that access to the biomaterial in the well is denied.
		@param well The current biowell
	*/
	public void setCurrentWell(BioWell well)
	{
		this.currentWell = well;
	}

	private String getCurrentNodeName()
	{
		return "well." + currentWell.getId();
	}

	private String getCurrentTitlePrefix()
	{
		return "[" + currentWell.getCoordinate() + "] ";
	}

}
