<%-- $Id: edit_protocol.jsp 6305 2013-08-09 09:10:46Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------


	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.Protocol"
	import="net.sf.basedb.core.ItemSubtype"
	import="net.sf.basedb.core.AnnotationType"
	import="net.sf.basedb.core.File"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.json.JsonUtil"
	import="net.sf.basedb.util.json.NameableConverter"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.edit.EditUtil"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="java.util.List"
	import="org.json.simple.JSONObject"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%
final Item itemType = Item.PROTOCOL;
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
try
{
	String title = null;
	Protocol protocol = null;
	boolean readCurrentSubtype = true;
	int currentSubtypeId = 0;
	boolean readCurrentFile = true;
	File currentFile = null;
	ItemQuery<AnnotationType> parameterQuery = null;

	// Load recently used items
	List<File> recentFiles = (List<File>)cc.getRecent(dc, Item.FILE);

	if (itemId == 0)
	{
		title = "Create protocol";

		currentSubtypeId = Values.getInt(request.getParameter("subtype_id"));
		if (currentSubtypeId == 0) 
		{
			int recentSubtypeId = Values.getInt(cc.getRecent(Item.ITEMSUBTYPE.name(), 0));
			currentSubtypeId = Values.getInt(cc.getPropertyValue("itemSubtype"), recentSubtypeId);
		}
		cc.removeObject("item");
	}
	else
	{
		protocol = Protocol.getById(dc, itemId);
		protocol.checkPermission(Permission.WRITE);
		
		parameterQuery = protocol.getParameters();
		parameterQuery.include(Include.ALL);
		parameterQuery.order(Orders.asc(Hql.property("name")));
		cc.setObject("item", protocol);
		title = "Edit protocol -- " + HTML.encodeTags(protocol.getName());
		try
		{
			ItemSubtype subtype = protocol.getItemSubtype();
			if (subtype != null) currentSubtypeId = subtype.getId();
		}
		catch (PermissionDeniedException ex)
		{
			readCurrentSubtype = false;
		}
		try
		{
			currentFile = protocol.getFile();
		}
		catch (PermissionDeniedException ex)
		{
			readCurrentFile = false;
		}
	}
	
	// Query to retrieve protocol types
	final ItemQuery<ItemSubtype> subtypesQuery = Base.getSubtypesQuery(itemType);
	subtypesQuery.include(Include.ALL);

	JSONObject jsonParameters = new JSONObject();
	jsonParameters.put("itemType", "ANNOTATIONTYPE");
	if (parameterQuery != null)
	{
		jsonParameters.put("items", JsonUtil.toArray(parameterQuery.iterate(dc), new NameableConverter()));
	}

	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, GuiContext.item(itemType), protocol);
	ExtensionsInvoker invoker = EditUtil.useEditExtensions(jspContext);
	%>
	<base:page type="popup" title="<%=title%>" id="edit-page">
	<base:head scripts="tabcontrol-2.js,linkitems-2.js,subtypes.js,~protocols.js" styles="tabcontrol.css">
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
	</base:head>
	<base:body>
		<h1><%=title%> <base:help tabcontrol="settings" /></h1>
		<form action="index.jsp?ID=<%=ID%>" method="post" name="protocol">
		<input type="hidden" name="cmd" value="UpdateItem">

		<div id="page-data" class="datacontainer"
			data-protocol-filetype-id="<%=SystemItems.getId(File.PROTOCOL)%>"
		></div>

		<t:tabcontrol id="settings"
			subclass="content dialogtabcontrol"
			position="bottom"  remember="<%=protocol != null%>" 
			extensions="<%=invoker%>">
		<t:tab id="info" title="Protocol" helpid="protocol.edit">
			<table class="fullform input100 smaller">
			<tr>
				<th>Name</th>
				<td><input class="text required auto-init" data-auto-init="<%=protocol == null ? "focus-select" : "focus" %>"
					type="text" name="name" 
					value="<%=HTML.encodeTags(protocol == null ? Values.getString(cc.getPropertyValue("name"), "New protocol") : protocol.getName())%>" 
					maxlength="<%=Protocol.MAX_NAME_LENGTH%>"></td>
				<td></td>
			</tr>
			<tr>
				<th>Type</th>
				<td>
					<select name="subtype_id" id="subtype_id"
						<%=!readCurrentSubtype ? "disabled readonly class=\"disabled selectionlist\"" : "class=\"selectionlist\""%>
						>
					<%
					if (!readCurrentSubtype)
					{
						%>
						<option value="-1">- denied -
						<%
					}
					else
					{
						%>
						<option value="0">-none-
						<%
						for (ItemSubtype subtype : subtypesQuery.list(dc))
						{
							int id = subtype.getId();
							if (id != currentSubtypeId && subtype.isRemoved()) continue;
							%>
							<option value="<%=id == currentSubtypeId && protocol != null ? -id : id%>" 
								<%=id == currentSubtypeId ? "selected" : ""%>
								title="<%=HTML.encodeTags(subtype.getDescription()) %>"
								><%=HTML.encodeTags(subtype.getName())%>
							<%
						}
					}
					%>
					</select>
				</td>
				<td></td>
			</tr>
			<tr>
				<th>External ID</th>
				<td><input class="text" type="text" name="external_id" 
					value="<%=HTML.encodeTags(protocol == null ? Values.getString(cc.getPropertyValue("externalId"), "") : protocol.getExternalId())%>" 
					maxlength="<%=Protocol.MAX_EXTERNAL_ID_LENGTH%>"></td>
				<td></td>
			</tr>
			<tr>
				<th>File</th>
				<td>
					<base:select 
						id="file_id"
						clazz="selectionlist"
						required="false"
						current="<%=currentFile%>"
						denied="<%=!readCurrentFile%>"
						recent="<%=recentFiles%>"
						newitem="<%=protocol == null%>"
					/>
				</td>
				<td></td>
			</tr>
			<tr class="dynamic">
				<th>Description</th>
				<td>
					<textarea class="text" rows="6" name="description" id="description"
						><%=HTML.encodeTags(protocol == null ? cc.getPropertyValue("description") : protocol.getDescription())%></textarea>
				</td>
				<td style="width: 20px;">
					<base:zoom textarea="description" title="Description" />
				</td>
			</tr>
			<%
			if (sc.getActiveProjectId() != 0 && protocol == null && sc.getActiveProjectPermissions().contains(Permission.WRITE))
			{
				%>
				<tr>
					<th></th>
					<td style="white-space: nowrap; overflow: hidden; padding-bottom: 2px;">
						<input type="checkbox" id="setAsProjectDefault" name="setAsProjectDefault" value="1" 
							checked><label for="setAsProjectDefault">Add as project default</label><br>
						<input type="checkbox" id="replaceExistingDefault" name="replaceExistingDefault" value="1"
							checked><label for="replaceExistingDefault">Replace existing default</label>
							<span id="currentDefault" style="font-style: italic;"></span>
					</td>
					<td></td>
				</tr>
				<%
			}
			%>
			</table>
		</t:tab>
		
		<t:tab id="parameters" title="Parameters" 
			helpid="protocol.edit.parameters">
			<table class="fullform input100">
			<tr class="big">
				<th>Annotation types</th>
				<td>
				<div class="selectionlist">
					<table>
					<tr>
						<td>
						<select name="annotationTypes" id="annotationTypes" 
							class="auto-init" 
							data-auto-init="link-container"
							data-initial-items="[<%=HTML.encodeTags(jsonParameters.toJSONString()) %>]"
							size="20" multiple>
						</select>
						</td>
						<td style="vertical-align: top;">
							<base:buttongroup vertical="true">
								<base:button 
									id="btnAddProtocolParameters"
									subclass="leftaligned"
									style="width: 15em;"
									title="Add annotation types&hellip;" 
									tooltip="Add annotation types"
									/>
								<base:button 
									subclass="leftaligned auto-init"
									data-auto-init="remove-link"
									data-list-id="annotationTypes"
									style="width: 15em;"
									title="Remove" 
									tooltip="Remove the selected annotation types"
								/>
							</base:buttongroup>
						</td>
					</tr>
					</table>
				</div>
				</td>
			</tr>
			<tr class="dynamic">
				<th></th>
				<td></td>
			</tr>
			</table>
		</t:tab>
		
		<t:tab id="annotations" title="Annotations" helpid="annotations.edit">
			<jsp:include page="../../common/annotations/annotate_frameset.jsp">
				<jsp:param name="item_type" value="<%=itemType.name()%>" />
				<jsp:param name="item_id" value="<%=itemId%>" />
				<jsp:param name="ID" value="<%=ID%>" />
			</jsp:include>
		</t:tab>
		</t:tabcontrol>
		</form>
		
		<div class="legend">
			<base:icon image="required.png" />= required information
		</div>

		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnSave" title="Save" />
			<base:button id="close" title="Cancel" />
		</base:buttongroup>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
