<%-- $Id: list_bioplates.jsp 7269 2017-01-11 12:48:14Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Johan Enell, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.AnnotationType"
	import="net.sf.basedb.core.AnnotationSet"
	import="net.sf.basedb.core.Annotation"
	import="net.sf.basedb.core.BioPlate"
	import="net.sf.basedb.core.PlateGeometry"
	import="net.sf.basedb.core.BioWell"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.Type"
	import="net.sf.basedb.core.ItemSubtype"
	import="net.sf.basedb.core.ItemResultIterator"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Nameable"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.PluginDefinition"
	import="net.sf.basedb.core.Metadata"
	import="net.sf.basedb.core.Metadata.PropertyPath"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.query.Restrictions"
	import="net.sf.basedb.core.query.Expressions"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.core.plugin.Plugin"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.snapshot.AnnotationLoaderUtil"
	import="net.sf.basedb.core.snapshot.AnnotationTypeFilter"
	import="net.sf.basedb.core.snapshot.AnnotationSnapshot"
	import="net.sf.basedb.core.snapshot.AnnotationSetSnapshot"
	import="net.sf.basedb.core.snapshot.SnapshotManager"
	import="net.sf.basedb.util.Enumeration"
	import="net.sf.basedb.util.ShareableUtil"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.ModeInfo"
	import="net.sf.basedb.clients.web.PermissionUtil"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.renderer.PrefixSuffixRenderer"
	import="net.sf.basedb.clients.web.extensions.toolbar.ToolbarUtil"
	import="net.sf.basedb.clients.web.extensions.list.ListColumnUtil"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="net.sf.basedb.util.Values"
	import="java.util.Iterator"
	import="java.util.List"
	import="java.util.Map"
	import="java.util.Date"
	import="java.util.ArrayList"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%!
	private static final Item itemType = Item.BIOPLATE;
	private static final GuiContext guiContext = new GuiContext(itemType, GuiContext.Type.LIST);

	private static Enumeration<String, String> bioMaterialTypes = new Enumeration<String, String>();
	static
	{
		bioMaterialTypes.add("", "-any-");
		bioMaterialTypes.add(Integer.toString(Item.SAMPLE.getValue()), Item.SAMPLE.toString());
		bioMaterialTypes.add(Integer.toString(Item.EXTRACT.getValue()), Item.EXTRACT.toString());
	}
%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, Permission.DENIED, itemType);
final String ID = sc.getId();
final boolean createPermission = sc.hasPermission(Permission.CREATE, itemType);
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);

final ModeInfo mode = ModeInfo.get(request.getParameter("mode"));
final String callback = request.getParameter("callback");
final String title = mode.generateTitle("bioplate", "bioplates");
final DbControl dc = sc.newDbControl();
ItemResultIterator<BioPlate> bioplates = null;
List<AnnotationLoaderUtil> annotationLoaders = new ArrayList<AnnotationLoaderUtil>();
try
{
	final ItemQuery<AnnotationType> annotationTypeQuery = Base.getAnnotationTypesQuery(itemType);
	SnapshotManager manager = new SnapshotManager();
	for (AnnotationType at : annotationTypeQuery.list(dc))
	{
		annotationLoaders.add(new AnnotationLoaderUtil(dc, manager, at));
	}

	final ItemQuery<ItemSubtype> subtypesQuery = Base.getSubtypesQuery(null);
	subtypesQuery.restrict(
		Restrictions.in(
			Hql.property("itemType"), 
			Expressions.integer(Item.SAMPLE.getValue()), Expressions.integer(Item.EXTRACT.getValue())
		));
	
	final ItemQuery<PlateGeometry> geometryQuery = PlateGeometry.getQuery();
	geometryQuery.order(Orders.asc(Hql.property("name")));
	geometryQuery.setCacheResult(true);
	
	Map<Plugin.MainType, Integer> pluginCount = PluginDefinition.countPlugins(dc, guiContext);
	try
	{
		final ItemQuery<BioPlate> query = Base.getConfiguredQuery(dc, cc, true, BioPlate.getQuery(), mode);
		bioplates = query.iterate(dc);
	}
	catch (Throwable t)
	{
		cc.setMessage(t.getMessage());
		t.printStackTrace();
	}
	int numListed = 0;
	Formatter<Date> dateFormatter = FormatterFactory.getDateFormatter(sc);
	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, guiContext, null);
	ExtensionsInvoker invoker = ToolbarUtil.useExtensions(jspContext);
	ExtensionsInvoker columnsInvoker = ListColumnUtil.useExtensions(jspContext);
	%>
	<base:page title="<%=title==null ? "Bioplates" : title%>" type="<%=mode.getPageType()%>" id="list-page">
	<base:head scripts="table.js,~bioplates.js" styles="table.css,toolbar.css">
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
	</base:head>	
	<base:body>
		<h1><%=title==null ? "Bioplates" : title%></h1>
		<div class="content">
		<tbl:table 
			id="bioplates"
			columns="<%=cc.getSetting("columns")%>"
			sortby="<%=cc.getSortProperty()%>" 
			direction="<%=cc.getSortDirection()%>"
			action="index.jsp"
			sc="<%=sc%>"
			item="<%=itemType%>"
			filterrows="<%=cc.getFilterRows()%>"
			subclass="fulltable"
			>
			<tbl:hidden 
				name="mode" 
				value="<%=mode.getName()%>" 
			/>
			<tbl:hidden 
				name="callback" 
				value="<%=callback%>" 
				skip="<%=callback == null%>" 
			/>
			<tbl:columndef 
				id="name"
				property="name"
				datatype="string"
				title="Name"
				sortable="true" 
				filterable="true"
				exportable="true"
				show="always" 
			/>
			<tbl:columndef 
				id="id"
				clazz="uniquecol"
				property="id"
				datatype="int"
				title="ID"
				sortable="true"
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="barcode"
				property="barcode"
				datatype="string"
				title="Barcode"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="destroyed"
				property="destroyed"
				datatype="boolean"
				title="Destroyed"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="eventDate"
				property="eventDate"
				datatype="date"
				title="Created"
				sortable="true" 
				filterable="true"
				exportable="true"
				formatter="<%=dateFormatter%>"
			/>
			<tbl:columndef 
				id="entryDate"
				property="entryDate"
				datatype="date"
				title="Registered"
				sortable="true" 
				filterable="true"
				exportable="true"
				formatter="<%=dateFormatter%>"
			/>
			<tbl:columndef
				id="freeWells"
				property="freeWells"
				datatype="int"
				title="Free (total) wells"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<%
			Enumeration<String, String> geometries = new Enumeration<String, String>();
			ItemResultList<PlateGeometry> plateGeometries = geometryQuery.list(dc);
			for (PlateGeometry pg : plateGeometries)
			{
				geometries.add(Integer.toString(pg.getId()), HTML.encodeTags(pg.getName()));
			}
			%>
			<tbl:columndef 
				id="plateGeometry"
				property="plateGeometry"
				sortproperty="plateGeometry.name"
				exportproperty="plateGeometry.name:string"
				datatype="int"
				enumeration="<%=geometries%>"
				title="Geometry"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="bioPlateType"
				property="bioPlateType.name"
				datatype="string"
				title="Bioplate type"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef
				id="bioMaterialType"
				property="bioPlateType.bioMaterialType"
				title="Biomaterial type"
				datatype="int"
				sortable="true" 
				filterable="true" 
				exportable="true"
				enumeration="<%=bioMaterialTypes%>"
			/>
			<tbl:columndef 
				id="bioMaterialSubtype"
				property="bioPlateType.itemSubtype"
				sortproperty="bioPlateType.itemSubtype.name"
				exportproperty="bioPlateType.itemSubtype.name:string"
				datatype="int"
				enumeration="<%=Enumeration.fromItems(subtypesQuery.list(dc), "-any-")%>"
				title="Biomaterial subtype"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="freezer"
				property="freezer.name"
				datatype="string"
				title="Storage location"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="section"
				property="section"
				datatype="string"
				title="Section"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="tray"
				property="tray"
				datatype="string"
				title="Tray"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="position"
				property="position"
				datatype="string"
				title="Position"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="owner"
				property="owner.name"
				datatype="string"
				title="Owner"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="description"
				property="description"
				datatype="string"
				title="Description" 
				sortable="true" 
				filterable="true" 
				exportable="true"
			/>			
			<%
			for (AnnotationLoaderUtil loader : annotationLoaders)
			{
				AnnotationType at = loader.getAnnotationType();
				Formatter formatter = FormatterFactory.getTypeFormatter(sc, at.getValueType());
				Enumeration<String, String> annotationEnum = null;
				if (at.isEnumeration())
				{
					annotationEnum = new Enumeration<String, String>();
					if (!at.getDisplayAsList()) annotationEnum.add("", "-none-");
					List<?> values = at.getValues();
					for (Object value : values)
					{
						String encoded = formatter.format(value);
						annotationEnum.add(encoded, encoded);
					}
				}
				%>
				<tbl:columndef 
					id="<%="at"+at.getId()%>"
					title="<%=HTML.encodeTags(at.getName())+" [A]"%>" 
					property="<%="#"+at.getId()%>"
					annotation="true"
					datatype="<%=at.getValueType().getStringValue()%>"
					enumeration="<%=annotationEnum%>"
					smartenum="<%=at.getDisplayAsList() %>"
					sortable="<%=at.getMultiplicity() == 1%>" 
					filterable="true" 
					exportable="true"
					formatter="<%=formatter%>"
					unit="<%=at.getDefaultUnit()%>"
				/>
				<%
			}
			%>
			<tbl:columndef
				id="permission"
				title="Permission"
			/>
			<tbl:columndef
				id="sharedTo"
				title="Shared to"
				filterable="true"
				filterproperty="!sharedTo.name"
				datatype="string"
			/>
			<tbl:columndef 
				id="xt-columns" 
				extensions="<%=columnsInvoker%>" 
				jspcontext="<%=jspContext%>" 
			/>
			<div class="panelgroup bg-filled-50 bottomborder">
				<tbl:toolbar
					visible="<%=mode.hasToolbar()%>"
					subclass="bottomborder"
					>
					<tbl:button 
						id="btnNewItem"
						disabled="<%=!createPermission%>" 
						image="new.png" 
						title="New&hellip;" 
						tooltip="<%=createPermission ? "Create new bioplate" : "You do not have permission to create bioplate"%>" 
					/>
					<tbl:button 
						id="btnDeleteItems"
						image="delete.png"
						title="Delete" 
						tooltip="Delete the selected items" 
					/>
					<tbl:button 
						id="btnRestoreItems"
						image="restore.png"
						title="Restore" 
						tooltip="Restore the selected (deleted) items"
					/>
					<tbl:button 
						id="btnShareItems"
						image="share.png"
						title="Share&hellip;" 
						tooltip="Share the selected items"
					/>
					<tbl:button 
						id="btnSetOwner"
						image="take_ownership.png"
						title="Set owner&hellip;"
						tooltip="Change owner of the selected items"
					/>
					<tbl:button 
						id="btnColumns"
						image="columns.png" 
						title="Columns&hellip;" 
						tooltip="Show, hide and re-order columns" 
					/>
					<tbl:button 
						id="btnImport"
						data-plugin-type="IMPORT"
						image="import.png" 
						title="Import&hellip;" 
						tooltip="Import data" 
						visible="<%=pluginCount.containsKey(Plugin.MainType.IMPORT)%>"
					/>
					<tbl:button 
						id="btnExport"
						data-plugin-type="EXPORT"
						image="export.png" 
						title="Export&hellip;" 
						tooltip="Export data" 
						visible="<%=pluginCount.containsKey(Plugin.MainType.EXPORT)%>"
					/>
					<tbl:button 
						id="btnRunPlugin"
						data-plugin-type="OTHER"
						image="runplugin.png" 
						title="Run plugin&hellip;" 
						tooltip="Run a plugin" 
						visible="<%=pluginCount.containsKey(Plugin.MainType.OTHER)%>"
					/>
					<ext:render extensions="<%=invoker%>" context="<%=jspContext%>" 
						wrapper="<%=new PrefixSuffixRenderer(jspContext, "<td>", "</td>") %>"/>
				</tbl:toolbar>
				<tbl:panel>
					<tbl:presetselector />
					<tbl:navigator
						page="<%=cc.getPage()%>" 
						rowsperpage="<%=cc.getRowsPerPage()%>" 
						totalrows="<%=bioplates == null ? 0 : bioplates.getTotalCount()%>" 
						visible="<%=mode.hasNavigator()%>"
					/>
				</tbl:panel>
			</div>
			<tbl:data>
				<tbl:headers>
					<tbl:headerrow>
						<tbl:header colspan="3" />
						<tbl:columnheaders />
					</tbl:headerrow>
					<%
					int numFilters = cc.getNumPropertyFilters();
					int numRows = cc.getFilterRows();
					for (int filterNo = 0; filterNo < numRows; filterNo++)
					{
						boolean lastRow = filterNo == numRows-1;
						%>
						<tbl:headerrow>
							<tbl:header subclass="index" />
							<tbl:header 
								subclass="check" 
								visible="<%=mode.hasCheck()%>"
								><base:icon 
									subclass="link table-check"
									image="check_uncheck.png" 
									tooltip="Toggle all (use CTRL, ALT or SHIFT to check/uncheck)" 
									visible="<%=lastRow%>"
								/></tbl:header>
							<tbl:header 
								subclass="check" 
								visible="<%=mode.hasRadio()%>"
								/>
							<tbl:header 
								subclass="icons" 
								visible="<%=mode.hasIcons()%>"
								>
								<base:icon
									subclass="link table-filter-row-action"
									image="add.png"
									tooltip="Add extra filter row"
									visible="<%=lastRow%>"
								/><base:icon
									subclass="link table-filter-row-action"
									image="remove.png"
									tooltip="Remove this filter row"
									visible="<%=numRows > 1 || numFilters > 0 %>"
									data-remove-row="<%=filterNo%>"
								/>
							</tbl:header>
							<tbl:propertyfilter row="<%=filterNo%>" />
						</tbl:headerrow>
						<%
					}
					%>
				</tbl:headers>
				<tbl:rows>
					<%
					if (cc.getMessage() != null)
					{
						%>
						<tbl:panel subclass="bg-filled-50">
							<div class="messagecontainer error"><%=cc.getMessage()%></div>
						</tbl:panel>
						<%
						cc.setMessage(null);
					}
					int index = cc.getPage()*cc.getRowsPerPage();
					int selectedItemId = cc.getId();
					if (bioplates != null)
					{
						PropertyPath<BioPlate, Integer> bmType = 
							Metadata.getInstance(BioPlate.class).getPropertyPath("bioPlateType.bioMaterialType", false);
						while (bioplates.hasNext())
						{
							BioPlate item = bioplates.next();
							int itemId = item.getId();
							boolean deletePermission = item.hasPermission(Permission.DELETE);
							boolean sharePermission = item.hasPermission(Permission.SET_PERMISSION);
							boolean writePermission = item.hasPermission(Permission.WRITE);
							String tooltip = mode.isSelectionMode() ? 
									"Select this item" : "View this item" + (writePermission ? " (use CTRL, ALT or SHIFT to edit)" : "");
							String name = HTML.encodeTags(item.getName());
							index++;
							numListed++;
							%>
							<tbl:row>
								<tbl:header 
									clazz="index"
									><%=index%></tbl:header>
								<tbl:header 
									clazz="check" 
									visible="<%=mode.hasCheck()%>"
									><input 
										type="checkbox" 
										name="<%=itemId%>" 
										value="<%=itemId%>" 
										title="<%=name%>" 
										<%=cc.getSelected().contains(itemId) ? "checked" : ""%> 
									></tbl:header>
								<tbl:header 
									clazz="check" 
									visible="<%=mode.hasRadio()%>"
									><input 
										type="radio" 
										name="item_id" 
										value="<%=itemId%>" 
										title="<%=name%>" 
										<%=selectedItemId == itemId ? "checked" : ""%>
									></tbl:header>
								<tbl:header 
									clazz="icons" 
									visible="<%=mode.hasIcons()%>"
									><base:icon 
										image="deleted.png"
										id="<%="delete."+itemId %>"
										subclass="<%=deletePermission ? "table-delete-item" : "disabled" %>"
										data-item-id="<%=itemId%>"
										tooltip="This item has been scheduled for deletion" 
										visible="<%=item.isRemoved()%>"
									/><base:icon 
										image="shared.png" 
										id="<%="share."+itemId %>"
										subclass="<%=sharePermission ? "table-share-item" : "disabled" %>"
										data-item-id="<%=itemId%>"
										tooltip="This item is shared to other users, groups and/or projects" 
										visible="<%=item.isShared()%>"
									/>&nbsp;</tbl:header>
								<tbl:cell column="name"><div 
									class="link table-item"
									data-item-id="<%=itemId%>"
									data-no-edit="<%=writePermission ? 0 : 1 %>" 
									tabindex="0"
									title="<%=tooltip%>"><%=name%></div></tbl:cell>
								<tbl:cell column="id"><%=item.getId()%></tbl:cell>
								<tbl:cell column="barcode"><%=HTML.encodeTags(item.getBarcode())%></tbl:cell>
								<tbl:cell column="destroyed"><%=item.isDestroyed()%></tbl:cell>
								<tbl:cell column="freeWells"><%=item.getFreeWells()%> (<%=item.getTotalWells()%>)</tbl:cell>
								<tbl:cell column="owner"
									><base:propertyvalue 
										item="<%=item%>" 
										property="owner"
										enableEditLink="<%=mode.hasEditLink()%>" 
										enablePropertyLink="<%=mode.hasPropertyLink()%>"
									/></tbl:cell>
								<tbl:cell column="plateGeometry"
									><base:propertyvalue 
										item="<%=item%>" 
										property="plateGeometry"
										enableEditLink="<%=mode.hasEditLink()%>" 
										enablePropertyLink="<%=mode.hasPropertyLink()%>"
									/></tbl:cell>
								<tbl:cell column="freezer">
									<base:propertyvalue 
										item="<%=item%>" 
										property="freezer"
									 	enableEditLink="<%=mode.hasEditLink()%>" 
										enablePropertyLink="<%=mode.hasPropertyLink()%>"
									/>
								</tbl:cell>
								<tbl:cell column="section"><%=HTML.encodeTags(item.getSection())%></tbl:cell>								
								<tbl:cell column="tray"><%=HTML.encodeTags(item.getTray())%></tbl:cell>								
								<tbl:cell column="position"><%=HTML.encodeTags(item.getPosition())%></tbl:cell>								
								<tbl:cell column="bioPlateType">
									<base:propertyvalue 
										item="<%=item%>" 
										property="bioPlateType"
									 	enableEditLink="<%=mode.hasEditLink()%>" 
										enablePropertyLink="<%=mode.hasPropertyLink()%>"
									/>
								</tbl:cell>
								<tbl:cell column="bioMaterialType"><%
									Integer i = bmType.getValue(dc, item);
									%><%=i == null ? "<i>- any -</i>" : Item.fromValue(i).toString() %></tbl:cell>
								<tbl:cell column="bioMaterialSubtype"><base:propertyvalue 
										item="<%=item%>" 
										property="bioPlateType.itemSubtype"
										enableEditLink="<%=mode.hasEditLink()%>" 
										enablePropertyLink="<%=mode.hasPropertyLink()%>"
										nulltext="<i>- any -</i>"
									/></tbl:cell>
								<tbl:cell column="eventDate" value="<%=item.getEventDate()%>" />
								<tbl:cell column="entryDate" value="<%=item.getEntryDate()%>" />
								<tbl:cell column="description"><%=HTML.encodeTags(item.getDescription())%></tbl:cell>								
								<%
								if (item.isAnnotated())
								{
									AnnotationSetSnapshot snapshot = manager.getSnapshot(dc, item.getAnnotationSet().getId());
									for (AnnotationLoaderUtil loader : annotationLoaders)
									{
										%>
										<tbl:cell 
											column="<%="at"+loader.getId()%>"
											><%
											if (loader.find(snapshot)) 
											{
												%><tbl:cellvalue 
													list="<%=loader.getValues()%>"
													suffix="<%=loader.getUnitSymbol()%>"
												/><%
											}
											%></tbl:cell>
										<%
									}
								}
								%>
								<tbl:cell column="permission"><%=PermissionUtil.getShortPermissions(item)%></tbl:cell>
								<tbl:cell column="sharedTo">
									<%
									Iterator<Nameable> sharees = ShareableUtil.getSharedTo(dc, item).iterator();
									while(sharees.hasNext())
									{
										Nameable n = sharees.next();
										if (mode.hasPropertyLink())
										{
											out.write(Base.getLinkedName(ID, n, false, mode.hasEditLink()));
										}
										else
										{
											out.write(HTML.encodeTags(n.getName()));
										}
										out.write(sharees.hasNext() ? ", " : "");
									}
									%>
								</tbl:cell>
								<tbl:xt-cells dc="<%=dc%>" item="<%=item%>">
									<tbl:cell column="xt-columns" />
								</tbl:xt-cells>
							</tbl:row>
							<%
							}
						}
					if (numListed == 0)
					{
						%>
						<tbl:panel subclass="bg-filled-50">
							<div class="messagecontainer note">
								<%=bioplates == null || bioplates.getTotalCount() == 0 ? "No plates were found" : "No plates on this page. Please select another page!" %>
							</div>	
						</tbl:panel>
						<%
					}
					%>
				</tbl:rows>
			</tbl:data>
		</tbl:table>
		</div>

		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnOk" title="Ok" visible="<%=mode.hasOkButton()%>" />
			<base:button id="close" title="Cancel" visible="<%=mode.hasCancelButton()%>" />
			<base:button id="close" title="Close" visible="<%=mode.hasCloseButton()%>" />
		</base:buttongroup>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (bioplates != null) bioplates.close();
	if (dc != null) dc.close();
}
%>