/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Annotate = function()
{
	var annotate = {};

	var MODIFIED = 'MODIFIED';
	var SYNCED = 'SYNCED';
	var CONVERTED = 'CONVERTED';
	var DELETE = 'DELETE';
	var INHERITED = 'INHERITED';
	
	var annotations;
	var parents;
	var selectedAnnotation;
	var selectedMultiIndex;
	

	annotate.initPage = function()
	{
		// Buttons (on standalone dialog)
		Buttons.addClickHandler('close', App.closeWindow);
		Buttons.addClickHandler('btnSave', annotate.save);
		
		// Parse the current annotation information which contain value type, current values, etc.
		annotations = Data.json('page-data', 'annotations');
		parents = Data.json('page-data', 'parents');
		
		// Loop through all annotations and build the selection list
		var numPrimary = 0;
		for (var i = 0; i < annotations.length; i++)
		{
			annotate.createAnnotationEntryInList(annotations[i], false);
			if (annotations[i].source == 'PRIMARY') numPrimary++;
		}
		if (numPrimary == 0) Doc.hide('primary-list');
		if (numPrimary == annotations.length) Doc.hide('inherited-list');

		// Event handlers for input fields
		Events.addEventHandler('STRING-input', 'change', annotate.valueOnChange);
		Events.addEventHandler('STRING-input', 'blur', annotate.valueOnBlur);
		Events.addEventHandler('TEXT-input', 'change', annotate.valueOnChange);
		Events.addEventHandler('TEXT-input', 'blur', annotate.valueOnBlur);
		Buttons.addClickHandler('TEXT-zoom', annotate.openTextZoom);
		Events.addEventHandler('INT-input', 'keypress', Events.integerOnly);
		Events.addEventHandler('INT-input', 'change', annotate.valueOnChange);
		Events.addEventHandler('INT-input', 'blur', annotate.valueOnBlur);
		Events.addEventHandler('LONG-input', 'keypress', Events.integerOnly);
		Events.addEventHandler('LONG-input', 'change', annotate.valueOnChange);
		Events.addEventHandler('LONG-input', 'blur', annotate.valueOnBlur);
		Events.addEventHandler('FLOAT-input', 'keypress', Events.numberOnly);
		Events.addEventHandler('FLOAT-input', 'change', annotate.valueOnChange);
		Events.addEventHandler('FLOAT-input', 'blur', annotate.valueOnBlur);
		Events.addEventHandler('DOUBLE-input', 'keypress', Events.numberOnly);
		Events.addEventHandler('DOUBLE-input', 'change', annotate.valueOnChange);
		Events.addEventHandler('DOUBLE-input', 'blur', annotate.valueOnBlur);
		Events.addEventHandler('DATE-input', 'change', annotate.valueOnChange);
		Events.addEventHandler('DATE-input', 'blur', annotate.valueOnBlur);
		Buttons.addClickHandler('DATE-calendar', annotate.openCalendar);
		Events.addEventHandler('TIMESTAMP-input', 'change', annotate.valueOnChange);
		Events.addEventHandler('TIMESTAMP-input', 'blur', annotate.valueOnBlur);
		Buttons.addClickHandler('TIMESTAMP-calendar', annotate.openCalendar);
		Events.addEventHandler('BOOLEAN-input-NULL', 'click', annotate.valueOnChange);
		Events.addEventHandler('BOOLEAN-input-TRUE', 'click', annotate.valueOnChange);
		Events.addEventHandler('BOOLEAN-input-FALSE', 'click', annotate.valueOnChange);
		Events.addEventHandler('ENUM-SINGLE-input', 'change', annotate.valueOnChange);
		Events.addEventHandler('ENUM-MULTIPLE-input', 'change', annotate.valueOnChange);

		// Buttons for multi-valued annotations
		Buttons.addClickHandler('btnMultiAdd', annotate.addMultiValue);
		Buttons.addClickHandler('btnMultiRemove', annotate.removeMultiValue);
		Events.addEventHandler('multi-values', 'click', annotate.multiValueOnClick);
		
		
		// The category list box is used to filter visible annotations
		Events.addEventHandler('categories', 'change', annotate.categoryOnChange);
		annotate.categoryOnChange();
		
		// Unit selection
		Events.addEventHandler('unit', 'change', annotate.unitOnChange);
		
		// Check/uncheck all
		Events.addEventHandler('check-all-primary', 'click', annotate.checkAllPrimary);
		Events.addEventHandler('check-all-inherited', 'click', annotate.checkAllInherited);
		
		// Sync modified clone
		Events.addEventHandler('sync-modified', 'click', annotate.syncSelectedClone);
		Buttons.addClickHandler('btnSync', annotate.syncAllCheckedClones);
		
		// Convert between inherited/cloned
		Events.addEventHandler('convert-to-cloned', 'click', annotate.convertSelectedToCloned);
		Buttons.addClickHandler('btnClone', annotate.convertAllCheckedToCloned);
		Buttons.addClickHandler('btnUnclone', annotate.convertAllCheckedToInherited);
		
		// Delete annotations
		Buttons.addClickHandler('btnDelete', annotate.deleteAllChecked);
		// Add inherited annotations
		Buttons.addClickHandler('btnAdd', annotate.addInheritedAnnotations);
		Events.addEventHandler('btnAdd', 'remove-annotation', annotate.onRemoveInheritedAnnotation);
		Events.addEventHandler('btnAdd', 'inherit-annotation', annotate.onAddInheritedAnnotation);
		
		// Select the initial annotation
		var annotationTypeId = Data.int('page-data', 'annotation-type-id');
		var annotationId = Data.int('page-data', 'annotation-id');
		if (annotationId)
		{
			annotate.selectAnnotation(annotationTypeId+'-'+annotationId);
		}
		else if (annotationTypeId) 
		{
			annotate.selectAnnotation(annotationTypeId)
		}
		else
		{
			annotate.selectAnnotation(null);
		}
	}
	
	/**
		Create an entry for the given annotation/annotation type
		in the list.
	*/
	annotate.createAnnotationEntryInList = function(entry, autoSort)
	{
		var ann = entry.annotation;
		var at = entry.annotationType;
		
		var div = document.createElement('div');
		div.className = 'param interactable';
		if (!entry.annotatePermission) div.className += ' disabled';
		div.id = entry.id;
		div.title = Strings.encodeTags(at.name);
		
		var html = '';
		var icon = annotate.getIcon(entry);
		
		if (entry.annotatePermission)
		{
			html += '<input type="checkbox" style="float: right;" id="check.'+entry.id+'">';
		}
		html += '<span class="icon">';
		html += '<img id="icon.'+entry.id+'" src="../../images/'+icon+'">';
		html += '</span>';
		// DEBUG!!
		//html += entry.source;
		
		html += '<span class="label">';
		if (at.removed) html += '<img src="../../images/deleted.png">';
		if (at.protocolParameter) html += '<img src="../../images/parameter.png">';
		html += Strings.encodeTags(at.name)+'</span>';
		html += '</div>';
		div.innerHTML = html;
		
		Events.addEventHandler(div, 'click', annotate.annotationOnClick);
		var list = Doc.element(entry.source == 'PRIMARY' ? 'primary-list' : 'inherited-list');
		var beforeChild = null;
		if (autoSort)
		{
			var childNo = 0;
			while (childNo < list.childNodes.length)
			{
				var child = list.childNodes[childNo];
				if (child.title && child.title > div.title)
				{
					beforeChild = child;
					break;
				}
				childNo++;
			}
		}
		list.insertBefore(div, beforeChild);
	}
	
	/**
		Event handler for the 'Categories' list. This will
		filter the main list to only include annotations in the select
		category. Special cases are 'all', 'protocol parameter' and 'uncategorized'
	 */
	annotate.categoryOnChange = function()
	{
		var frm = document.forms['annotations'];
		var categoryId = parseInt(frm.categories.value);
		
		for (var entryNo = 0; entryNo < annotations.length; entryNo++)
		{
			var entry = annotations[entryNo];
			var at = entry.annotationType;

			var show = false;
			if (entry.source != 'PRIMARY')
			{
				// INHERITED or CLONED annotations are visible unless DELETEd
				show = entry.modified != 'DELETE';
			}
			else if (categoryId == -1)
			{
				// show all
				show = true;
			}
			else if (categoryId == -2 && at.protocolParameter)
			{
				// show protocol parameters
				show = true;
			}
			else if (categoryId == 0 && at.categories.length == 0 && !at.protocolParameter)
			{
				// show uncategorized
				show = true;
			}
			else
			{
				// show the selected category
				for (var categoryNo = 0; categoryNo < at.categories.length; categoryNo++)
				{
					if (at.categories[categoryNo] == categoryId) 
					{
						show = true;
						break;
					}
				}
			}
			
			Doc.showHide(entry.id, show);
		}
	}
	
	/**
		Get the annotation entry for the given id.
		Returns null if not found.
	*/
	annotate.getEntryById = function(entryId)
	{
		for (var i = 0; i < annotations.length; i++)
		{
			if (annotations[i].id == entryId)
			{
				return annotations[i];
			}
		}
		return null;
	}
	
	
	/**
		Select the given annotation entry. This will mark the annotation 
		in the main list, display the input fields associated with it
		and populate the input fields with the current value(s).
	*/
	annotate.selectAnnotation = function(entryId)
	{
		if (selectedAnnotation)
		{
			Doc.removeClass(selectedAnnotation.id, 'selected');
		}

		var frm = document.forms['annotations'];
		// Mark the selected annotation and make sure the
		// category list doesn't hide it
		selectedAnnotation = annotate.getEntryById(entryId);
		selectedMultiIndex = -1;
		
		var classes = [];
		var valueElement;
		
		if (selectedAnnotation == null)
		{
			classes.push('none-selected');
			if (annotations.length == 0) 
			{
				classes.push('no-annotations');
			}
		}
		else
		{
			
			Doc.addClass(entryId, 'selected');
			if (Doc.element(entryId).style.display=='none')
			{
				// Make sure the entry is not hidden because a different
				// category is selected
				frm.categories.selectedIndex = 0;
				annotate.categoryOnChange();
			}
			Doc.element(entryId).scrollIntoView(false);
			
			var at = selectedAnnotation.annotationType;
			var ann = selectedAnnotation.annotation;
			
			annotate.showNameAndOtherInfo(at);
			classes.push('has-selected');
			classes.push(selectedAnnotation.source);
			if (at.description) classes.push('has-description');
			if (!selectedAnnotation.annotatePermission) classes.push('no-annotate-permission');
			
			if (selectedAnnotation.source != 'PRIMARY')
			{
				var inherited = selectedAnnotation.inherited;
				if (inherited)
				{
					var symbol = inherited.unitSymbol ? ' ' + inherited.unitSymbol : '';
					
					Doc.element('inherited-values').innerHTML = Strings.encodeTags(inherited.values.join(symbol+', ')+symbol);
					Doc.element('inherited-item').innerHTML = (selectedAnnotation.source == 'CLONED' ? 'Cloned from ' : 'Inherited from ') + Strings.encodeTags(inherited.from.name) + ' <span class="itemsubtype">('+inherited.from.type+')</span>';
					
					if (selectedAnnotation.source == 'CLONED' && inherited.lastUpdate > ann.lastUpdate)
					{
						classes.push('modified');
					}
					
				}
				else
				{
					classes.push('inherited-missing');
				}
			}
			
			if (!at.enumeration)
			{
				// Just display the correct input section
				classes.push(at.valueType);
				valueElement = frm[at.valueType+'-input'];
			}
			else
			{
				// We need to populate the selection list or
				// generate radiobuttons/checkboxes
				var symbol = at.defaultUnit ? ' ' + at.defaultSymbol : '';
				if (at.displayAsList)
				{
					classes.push(at.multiplicity == 1 ? 'ENUM-SINGLE' : 'ENUM-MULTIPLE');
					var list = Doc.element(at.multiplicity == 1 ? 'ENUM-SINGLE-input' : 'ENUM-MULTIPLE-input');
					annotate.initEnumList(list, at.enumeration, at.multiplicity == 1 && selectedAnnotation.source == 'PRIMARY', symbol);
					valueElement = list;					
				}
				else
				{
					classes.push('ENUM-RADIO-CHECKBOX');
					var tmp = annotate.initEnumCheckboxes(at.enumeration, at.multiplicity == 1, symbol, selectedAnnotation.source == 'PRIMARY');
					Doc.element('ENUM-RADIO-CHECKBOX').innerHTML = '';
					Doc.element('ENUM-RADIO-CHECKBOX').appendChild(tmp);
					valueElement = frm['check'];
				}
			}
		
			if (at.units)
			{
				classes.push('has-unit')
				annotate.initUnitList(at.units, ann && ann.unit || at.defaultUnit);
			}
			
			
			// Fill form with current values
			var values = ann.values.length > 0 || at.multiplicity != 1 ? ann.values : [''];
			if (valueElement.type && valueElement.type.indexOf('select') == 0)
			{
				// selection list
				Forms.selectListOptions(valueElement, values);
			}
			else if (valueElement.length)
			{
				// radio buttons or checkboxes
				if (at.multiplicity == 1)
				{
					Forms.checkRadio(valueElement, values[0]);
				}
				else
				{
					Forms.checkCheckBoxes(valueElement, values);
				}
			}
			else
			{
				// regular input field
				if (at.multiplicity == 1)
				{
					valueElement.value = values[0];
				}
				else
				{
					classes.push('multiple');
					// Fill the 'multi-values' list with the current values
					var multiValues = frm['multi-values'];
					multiValues.length = 0;
					for (var i = 0; i < values.length; i++) 
					{
						multiValues[multiValues.length] = new Option(values[i]);
					}
					valueElement.value = '';
				}
			}
		}
		Doc.element('selected-container').className = classes.join(' ');
		if (valueElement && valueElement.focus) valueElement.focus();
	}
	
	/**
		Display information about the annotation.
	*/
	annotate.showNameAndOtherInfo = function(at)
	{
		Doc.element('selected-name').innerHTML = Strings.encodeTags(at.name);
		Doc.element('selected-description').innerHTML = at.description || '';
		
		var typeSummary = at.valueType;
		if (at.enumeration)
		{
			if (at.multiplicity == 0)
			{
				typeSummary = "Select one or more";
			}
			else if (at.multiplicity == 1)
			{
				typeSummary = "Select one";
			}
			else
			{
				typeSummary = "Select 1‒"+at.multiplicity;
			}
		}
		else
		{
			if (at.minValue != null && at.maxValue != null)
			{
				typeSummary += ': '+at.minValue+'-'+at.maxValue;
			}
			else if (at.minValue != null)
			{
				typeSummary += ' &gt;= '+at.minValue;
			}
			else if (at.maxValue != null)
			{
				typeSummary += ' &lt;= '+at.maxValue;
			}
			if (at.multiplicity > 1)
			{
				typeSummary += '; max '+at.multiplicity+' values';
			}
		}
		Doc.element('selected-type').innerHTML = typeSummary;
	}
	
	/**
		Initialize the units list with the given units.
		'selected' is the id of the unit that should be
		selected.
	 */
	annotate.initUnitList = function(units, selected)
	{
		var unitList = Doc.element('unit');
		unitList.length = 0;
		for (var unitNo = 0; unitNo < units.length; unitNo++)
		{
			var unit = units[unitNo];
			var option = new Option(unit.symbol, unit.id, false, selected == unit.id);
			option.title = unit.description;
			unitList[unitList.length] = option;
		}
	}
	
	/**
		Fill the given <select> list with the given
		options. If 'notSpecified' is set, the first
		option will be 'not specified'.
	*/
	annotate.initEnumList = function(list, values, notSpecified, unit)
	{
		list.length = 0;
		if (notSpecified)
		{
			list[0] = new Option('- not specified -', '');
			list[0].className = 'not-specified';
		}
		for (var valueNo = 0; valueNo < values.length; valueNo++)
		{
			var value = values[valueNo];
			list[list.length] = new Option(value+unit, value);
		}
	}
	
	/**
		Generate checkboxes or radiobuttons for an enumeration
		annotation.
	*/
	annotate.initEnumCheckboxes = function(values, radio, unit, includeNotSpecified)
	{
		var tmp = document.createDocumentFragment();
		var ctlType = radio ? 'radio' : 'checkbox';
		if (includeNotSpecified && radio)
		{
			var noop = annotate.createRadioOrCheckBox(ctlType, '', '- not specified -');
			noop.className = 'not-specified';
			tmp.appendChild(noop);
		}
		for (var valueNo = 0; valueNo < values.length; valueNo++)
		{
			var value = values[valueNo];
			tmp.appendChild(annotate.createRadioOrCheckBox(ctlType, value, value+unit));
		}
		return tmp;
	}

	/**
		Create a <input> element that is either a radio och checkbox.
	*/
	annotate.createRadioOrCheckBox = function(type, value, text)
	{
		var ctl = document.createElement('input');
		ctl.type = type;
		ctl.name = 'check';
		ctl.value = value;
		Events.addEventHandler(ctl, 'click', annotate.valueOnChange);
		var lbl = document.createElement('label');
		lbl.appendChild(ctl);
		lbl.appendChild(document.createTextNode(text));
		lbl.appendChild(document.createElement('br'));
		return lbl;
	}
	
	/**
		Event handler for selecting annotations from the main list.
	*/
	annotate.annotationOnClick = function(event)
	{
		annotate.selectAnnotation(event.currentTarget.id);
	}
	
	/**
		Update the given annotation with a single value. All existing
		values are overwritten. If the value is not accepted an
		exception is thrown.
	*/
	annotate.setSingleValue = function(entry, value)
	{
		var values = [];
		if (value != null && value != '')
		{
			values[0] = value;
		}
		annotate.setMultipleValues(entry, values);
	}
	
	/**
		Update the given annotation with multiple values. All existing
		values are overwritten. If the values are not accepted an
		exception is thrown.
	*/
	annotate.setMultipleValues = function(entry, values)
	{
		var at = entry.annotationType;
		var ann = entry.annotation;
		
		if (at.multiplicity > 0 && values.length > at.multiplicity)
		{
			throw 'Max '+at.multiplicity+' values allowed';
		}
		ann.values = values;
		entry.modified = MODIFIED;
		annotate.updateIcon(entry);
	}
	
	/**
		Update/add/remove a single value in a multi-valued annotation. If the value
		is not accepted an exception is thrown.
	*/
	annotate.setIndexedValue = function(entry, index, value)
	{
		var ann = entry.annotation;
		var at = entry.annotationType;
		
		if (!value)
		{
			if (index < ann.values.length)
			{
				ann.values.splice(index, 1);
			}
		}
		else
		{
			if (index >= ann.values.length)
			{
				// Add a new value
				if (at.multiplicity > 0 && ann.values.length >= at.multiplicity)
				{
					throw 'Max '+at.multiplicity+' values allowed';
				}
				index = ann.values.length;
			}
			ann.values[index] = value;
		}
		entry.modified = MODIFIED;
		annotate.updateIcon(entry);
	}
	
	/**
		Change the unit of the given annotation.
	*/
	annotate.setUnit = function(annotation, unit)
	{
		annotation.annotation.unit = unit;
		annotation.modified = MODIFIED;
	}
	
	/**
		Event handler that is executed when an annotation values has changed.
		This new value should be stored back into the 'annotations' array.
		The exact procedure depends on the type of value.
	*/
	annotate.valueOnChange = function(event)
	{
		if (!selectedAnnotation) return;
		
		var at = selectedAnnotation.annotationType;
		var frm = document.forms['annotations'];
		var target = event.currentTarget;
		try
		{
			if (at.multiplicity == 1) 
			{
				// If only a single value is accepted the event target holds the new value
				annotate.setSingleValue(selectedAnnotation, target.value);
			}
			else if (at.enumeration)
			{
				// If the annotation is a multi-valued enumeration the input fields can be either a
				// group of checkboxes or a select control allowing multiple selections.
				if (target.type == 'checkbox') target = frm[target.name]; // An array of all checkboxes in the group
				var values = [];
				for (var i = 0; i < target.length; i++)
				{
					if (target[i].checked || target[i].selected) 
					{
						values[values.length] = target[i].value;
					}
				}
				annotate.setMultipleValues(selectedAnnotation, values);
			}
			else
			{
				// Otherwise we use the 'multi-values' control and update the currently
				// selected option (but note that we can't use 'selectedIndex' property of the
				// list control, since it may already point to the future selection
				var selectedIndex = frm['multi-values'].selectedIndex;
				if (selectedMultiIndex >= 0)
				{
					annotate.setIndexedValue(selectedAnnotation, selectedMultiIndex, target.value);
					var multiValues = frm['multi-values'];
					if (!target.value)
					{
						multiValues[selectedMultiIndex] = null;
						selectedMultiIndex = -1;
					}
					else
					{
						multiValues[selectedMultiIndex].text = target.value;
					}
				}
			}
		}
		catch (errorMessage)
		{
			event.preventDefault();
			Forms.showNotification(event.currentTarget, errorMessage);
		}
	}
	
	// To catch the case if 'new' was clicked but no value was entered
	annotate.valueOnBlur = function(event)
	{
		if (!selectedAnnotation || selectedMultiIndex < 0) return;
		
		var frm = document.forms['annotations'];
		
		if (event.currentTarget.value == '' && frm['multi-values'][selectedMultiIndex].isNewValue) 
		{
			// This will remove the new but empty value
			Events.sendChangeEvent(event.currentTarget);
		}
		else
		{
			frm['multi-values'][selectedMultiIndex].isNewValue = false;
		}
	}
	
	/**
		Event handler that handle changes to the selected unit.
	*/
	annotate.unitOnChange = function(event)
	{
		annotate.setUnit(selectedAnnotation, parseInt(event.currentTarget.value));
	}

	/**
		Toggle the checkbox for all primary annotations. Holding down
		CTRL/SHIFT/ALT will selected or deselect all.
	*/
	annotate.checkAllPrimary = function(event)
	{
		var specialKey = event.altKey || event.ctrlKey || event.shiftKey;
		var first = null;
		for (var i = 0; i < annotations.length; i++)
		{
			var entry = annotations[i];
			if (entry.source == 'PRIMARY')
			{
				var checkbox = Doc.element('check.'+entry.id);
				if (checkbox)
				{
					if (first == null) first = checkbox.checked;
					checkbox.checked = specialKey ? !first : !checkbox.checked;
				}
			}
		}
	}
	
	/**
		Toggle the checkbox for all inherited/cloned annotations. Holding down
		CTRL/SHIFT/ALT will selected or deselect all.
	*/
	annotate.checkAllInherited = function(event)
	{
		var specialKey = event.altKey || event.ctrlKey || event.shiftKey;
		var first = null;
		for (var i = 0; i < annotations.length; i++)
		{
			var entry = annotations[i];
			if (entry.source != 'PRIMARY')
			{
				var checkbox = Doc.element('check.'+entry.id);
				if (checkbox)
				{
					if (first == null) first = checkbox.checked;
					checkbox.checked = specialKey ? !first : !checkbox.checked;
				}
			}
		}
	}
	
	/**
		Call the given method for all annotations that
		have been checked. The method must take a single
		parameter which is the entry object with the
		annotation information.
		@return The return value of the method when called
			on the current selected annotation or null if the
			selected annotation is not checked
	 */
	annotate.forAllChecked = function(method)
	{
		var returnValue = null;
		var numChecked = 0;
		for (var i = 0; i < annotations.length; i++)
		{
			var entry = annotations[i];
			var checkbox = Doc.element('check.'+entry.id);
			if (checkbox && checkbox.checked)
			{
				numChecked++;
				var rv = method.call(null, entry);
				if (entry == selectedAnnotation) returnValue = rv;
			}
		}
		if (numChecked == 0)
		{
			Forms.showNotification('annotation-list', 'Please select annotations first...', null, 'pointer-left');
		}
		return returnValue;
	}
	
	/**
		Synchronizes checked CLONED annotations.
	*/
	annotate.syncAllCheckedClones = function()
	{
		if (annotate.forAllChecked(annotate.syncClone))
		{
			annotate.selectAnnotation(selectedAnnotation.id);
		}
	}
	
	/**
		Synchronizes the currently selected CLONED annotation
	 */
	annotate.syncSelectedClone = function()
	{
		annotate.syncClone(selectedAnnotation);
		annotate.selectAnnotation(selectedAnnotation.id);
	}
	
	/**
		Syncs a cloned value with the parent items value.
		@returns TRUE if the entry was synced, FALSE if not.
	*/
	annotate.syncClone = function(entry)
	{
		if (entry.source != 'CLONED') return false;
		if (!entry.inherited) return false;
		
		annotate.setMultipleValues(entry, entry.inherited.values);
		annotate.setUnit(entry, entry.inherited.unit);
		entry.annotation.lastUpdate = entry.inherited.lastUpdate;
		entry.modified = SYNCED;
		annotate.updateIcon(entry);
		
		return true;
	}
	
	
	/**
		Converts checked INHERITED annotations to CLONED annotations.
	*/
	annotate.convertAllCheckedToCloned = function()
	{
		if (annotate.forAllChecked(annotate.convertToCloned))
		{
			annotate.selectAnnotation(selectedAnnotation.id);
		}
	}

	/**
		Converts the selected annotation to a CLONED annotation.
	*/
	annotate.convertSelectedToCloned = function()
	{
		annotate.convertToCloned(selectedAnnotation);
		annotate.selectAnnotation(selectedAnnotation.id);
	}

	/**
		Converts an INHERITED annotation to a CLONED annotation.
	*/
	annotate.convertToCloned = function(entry)
	{
		if (entry.source != 'INHERITED') return false;
		
		var ann = entry.annotation;
		var inherited = entry.inherited;
		
		entry.source = 'CLONED';
		entry.modified = CONVERTED;
		ann.values = inherited.values;
		ann.unit = inherited.unit;
		annotate.updateIcon(entry);

		return true;
	}

	/**
		Converts checked CLONED annotations to INHERITED annotations.
	*/
	annotate.convertAllCheckedToInherited = function()
	{
		if (annotate.forAllChecked(annotate.convertToInherited))
		{
			annotate.selectAnnotation(selectedAnnotation.id);
		}
	}

	/**
		Converts a CLONED annotation to an INHERITED annotation.
		If the annotation is not CLONED or if the parent item
		is unknown, nothing is done.
	*/
	annotate.convertToInherited = function(entry)
	{
		if (entry.source != 'CLONED') return false;
		
		var ann = entry.annotation;
		var inherited = entry.inherited;
		if (!inherited) return false;
		
		entry.source = 'INHERITED';
		entry.modified = CONVERTED;
		ann.values = [];
		ann.unit = null;
		annotate.updateIcon(entry);
		return true;
	}

	/**
		Delete all checked annotations.
	*/
	annotate.deleteAllChecked = function()
	{
		if (annotate.forAllChecked(annotate.deleteAnnotation))
		{
			annotate.selectAnnotation(selectedAnnotation.source == 'PRIMARY' ? selectedAnnotation.id : null);
		}
	}
	
	/**
		Delete an annotation. PRIMARY annotations have all their values removed.
		INHERITED/CLONED annotations are deleted.
	*/
	annotate.deleteAnnotation = function(entry)
	{
		if (entry.source == 'PRIMARY')
		{
			entry.modified = MODIFIED;
			entry.annotation.values = [];
			entry.annotation.unit = null;
			annotate.updateIcon(entry);
		}
		else
		{
			entry.modified = DELETE;
			Doc.hide(entry.id);
		}
		return true;
	}
	
	/**
		Open a popup dialog for selecting parent annotations
		that should be inherited.
	*/
	annotate.addInheritedAnnotations = function(event)
	{
		var url = 'inherit.jsp?ID='+App.getSessionId();
		url += '&item_type='+Data.get('page-data', 'item-type');
		url += '&item_id='+Data.get('page-data', 'item-id');
		url += '&callback='+event.currentTarget.id;
		
		// Items marked for DELETE should show up in the dialog
		var deleted = [];
		for (var i = 0; i < annotations.length; i++)
		{
			var entry = annotations[i];
			if (entry.source != 'PRIMARY' && entry.modified == DELETE && entry.annotation.id)
			{
				deleted[deleted.length] = entry.annotation.id;
			}
		}
		url += '&deleted='+deleted.join(',');
		
		if (parents.length)
		{
			url += '&parents='+parents.join('&parents=');
		}
		
		Dialogs.openPopup(url, 'InheritAnnotations', 750, 500);
	}
	
	annotate.setParents = function(newParents)
	{
		parents = newParents;
	}
	
	/**
		Event handler for removing an inherited annotation.
	*/
	annotate.onRemoveInheritedAnnotation = function(event)
	{
		var inheritedId = event.detail.id;
		for (var i = 0; i < annotations.length; i++)
		{
			var entry = annotations[i];
			if (entry.inherited && entry.inherited.id == inheritedId)
			{
				annotate.deleteAnnotation(entry);
				return;
			}
		}
	}
	
	/**
		Event handler for adding an inherited annotation. Note
		that the annotation may be an annotation that is already
		inherited but has been marked for DELETE. In this case the
		DELETE should be cancelled.
	*/
	annotate.onAddInheritedAnnotation = function(event)
	{
		Doc.show('inherited-list');
		var inheritedId = event.detail.id;
		var clone = event.detail.clone;
		
		// Check if we have info about this annotation already
		for (var i = 0; i < annotations.length; i++)
		{
			var entry = annotations[i];
			if (entry.inherited && entry.inherited.id == inheritedId)
			{
				entry.modified = INHERITED;
				if (clone && entry.source != 'CLONED')
				{
					annotate.convertToCloned(entry);
				}
				else if (!clone && entry.source != 'INHERITED')
				{
					annotate.convertToInherited(entry);
				}
				Doc.show(entry.id);
				return;
			}
		}
		
		// Load info about this annotation
		var url = 'ajax.jsp?ID='+App.getSessionId();
		url += '&cmd=GetAnnotationInfoForInherit';
		url += '&item_id='+inheritedId;
		
		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, true);
		request.send(null);
		var callback = clone ? annotate.inheritInfoLoadedAndClone : annotate.inheritInfoLoaded;
		Ajax.setReadyStateHandler(request, callback, callback);
	}
	
	/**
		Callback that converts the inherited annotation to a cloned
		annotation.
	*/
	annotate.inheritInfoLoadedAndClone = function(request)
	{
		var entry = annotate.inheritInfoLoaded(request);
		annotate.convertToCloned(entry);
	}
	
	/**
		Callback after we have got information about an annotation
		that we should inherit.
	*/
	annotate.inheritInfoLoaded = function(request)
	{
		var response = JSON.parse(request.responseText);
		if (response.status != 'ok')
		{
			App.debug(request.responseText);
			return;
		}
		
		var entry = response.info;
		entry.modified = INHERITED;
		
		annotations[annotations.length] = entry;
		annotate.createAnnotationEntryInList(entry, true);
		return entry;
	}
	
	annotate.addMultiValue = function(event)
	{
		if (!selectedAnnotation) return;
		
		var at = selectedAnnotation.annotationType;

		var frm = document.forms['annotations'];
		var valueElement = frm[at.valueType+'-input'];
		
		var multiValues = frm['multi-values'];
		var errorMessage = null;
		try
		{
			// If one item is already selected, we create a new entry
			if (selectedMultiIndex >= 0)
			{
				// Add a new option
				errorMessage = annotate.setIndexedValue(selectedAnnotation, multiValues.length, '');
				if (!errorMessage)
				{
					var opt = new Option('<new>', '', false, true);
					opt.isNewValue = true;
					multiValues[multiValues.length] = opt;
					selectedMultiIndex = multiValues.length-1;
					valueElement.value = '';
				}
			}
			else if (valueElement.value != '')
			{
				// Add the current value
				errorMessage = annotate.setIndexedValue(selectedAnnotation, multiValues.length, valueElement.value);
				if (!errorMessage)
				{
					multiValues[multiValues.length] = new Option(valueElement.value, '', false, false);
					valueElement.value = '';
				}
			}
			valueElement.focus();
		}
		catch (errorMessage)
		{
			Forms.showNotification('btnMultiAdd', errorMessage, null, 'pointer-left');
		}
	}
	
	annotate.removeMultiValue = function()
	{
		if (!selectedAnnotation) return;

		var frm = document.forms['annotations'];
		var selectedIndex = frm['multi-values'].selectedIndex;
		
		if (selectedIndex >= 0)
		{
			// Clear the current value, and send a 'change' event to let the 
			// regular event handlers take care of the removal
			var at = selectedAnnotation.annotationType;
			var valueElement = frm[at.valueType+'-input'];
			valueElement.value = '';
			Events.sendChangeEvent(valueElement);
		}
	}
		
	/**
		Event handler for the 'multi-values' selection list. This should change
		the value that is currently being edited to the selected option.
	*/
	annotate.multiValueOnClick = function(event)
	{
		if (!selectedAnnotation) return;
		
		var at = selectedAnnotation.annotationType;
		var ann = selectedAnnotation.annotation;
		
		var frm = document.forms['annotations'];
		selectedMultiIndex = event.currentTarget.selectedIndex;
		if (selectedMultiIndex >= 0)
		{
			var valueElement = frm[at.valueType+'-input'];
			valueElement.value = ann.values[selectedMultiIndex];
			valueElement.focus();
		}
	}
	
	
	annotate.getIcon = function(entry)
	{
		var icon;
		var ann = entry.annotation;
		
		if (entry.source == 'PRIMARY')
		{
			// 'x' for has values, empty otherwise
			icon = (ann && ann.values.length > 0) ? 'notrequired_values.png' : 'notrequired_novalues.png';
		}
		else if (entry.source == 'INHERITED')
		{
			// Always the same 'x' with an anchor
			icon = 'inherited.png'
		}
		else
		{
			// Cloned annotations
			if (!entry.inherited)
			{
				// No parent item
				icon = 'cloned-noparent.png';
			}
			else if (entry.inherited.lastUpdate > ann.lastUpdate)
			{
				// The parent annotation has been updated
				icon = 'cloned-outofsync.png';
			}
			else
			{
				icon = 'cloned.png';
			}
		}
		return icon;
	}
	
	/**
		Update the icon for the annotation. The icon is different depending on if
		there are values or not for the annotation.
	*/
	annotate.updateIcon = function(entry)
	{
		var icon = annotate.getIcon(entry);
		var img = Doc.element('icon.'+entry.id);
		img.src = App.getRoot()+'images/'+icon;
	}
	
	/**
		Open the calendar dialog for the selected annotation.
	*/ 
	annotate.openCalendar = function(event)
	{
		var at = selectedAnnotation.annotationType;
		var title = at.name;
		var useTime = at.valueType == 'TIMESTAMP';
		var format = Data.get('page-data', useTime ? 'datetime-format' : 'date-format');
		Dialogs.openCalendar(at.valueType+'-input', title, format, useTime);
	}
	
	/**
		Open the text zoom dialog for the selected annotation.
	*/ 
	annotate.openTextZoom = function(event)
	{
		var at = selectedAnnotation.annotationType;
		var title = at.name;
		Dialogs.openZoom(at.valueType+'-input', title);
	}

	/**
		Copy all modified annotation values to the given form as hidden fields.
	*/
	annotate.saveModifiedAnnotationToForm = function(frm)
	{
		var modified = [];
		for (var entryNo = 0; entryNo < annotations.length; entryNo++)
		{
			var entry = annotations[entryNo];
			if (entry.annotatePermission && entry.modified)
			{
				var tmp = {};
				modified[modified.length] = tmp;
				tmp.source = entry.source;
				tmp.modified = entry.modified;
				tmp.annotationTypeId = entry.annotationType.id;
				if (entry.annotation)
				{
					tmp.values = entry.annotation.values;
					tmp.annotationId = entry.annotation.id;
					tmp.unitId = entry.annotation.unit;
					if (tmp.modified == MODIFIED && tmp.values.length == 0)
					{
						tmp.modified = DELETE;
					}
				}
				if (entry.inherited)
				{
					tmp.inheritedId = entry.inherited.id;
				}
			}
		}
		
		//alert(JSON.stringify(modified));
		
		if (frm.modifiedAnnotations)
		{
			frm.modifiedAnnotations.value = JSON.stringify(modified);
		}
		else
		{
			Forms.addHidden(frm, 'modifiedAnnotations', JSON.stringify(modified));
		}
		
	}
	
	/**
		Save the annotations in standalone mode.
	*/
	annotate.save = function()
	{
		var frm = document.forms['modified'];
		annotate.saveModifiedAnnotationToForm(frm);
		frm.submit();
	}
	
	return annotate;
}();

Doc.onLoad(Annotate.initPage);


