/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var BioAssays = function()
{
	var bioassays = {};
	
	/**
		Initialize the page.
	*/
	bioassays.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', bioassays.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabActivateListener('settings.annotations', Annotations.onSwitchToAnnotationsTab);
			TabControl.addTabValidator('settings.info', bioassays.validateBioAssay);
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'ROOTRAWBIOASSAY', 'item-id': itemId};
			
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			
			TabControl.addTabActivateListener('main.annotations', AnnotationsList.loadOnce);
		}
	}
	
	bioassays.initElements = function(element, autoInit)
	{
		 if (autoInit == 'edit-experimental-factor')
		{
			Buttons.addClickHandler(element, bioassays.editExperimentalFactor);
		}
	}


	bioassays.editExperimentalFactor = function(event)
	{
		var url = App.getRoot() + 'common/annotations/annotate.jsp?ID='+App.getSessionId();
		url += '&item_type='+Data.get(event.currentTarget, 'item-type');
		url += '&item_id='+Data.get(event.currentTarget, 'item-id');
		url += '&annotationtype_id='+Data.get(event.currentTarget, 'annotation-type-id');
		url += '&annotation_id='+Data.get(event.currentTarget, 'annotation-id');
		url += '&standalone=1';
		Dialogs.openPopup(url, 'EditAnnotation', 750, 500);
	}
	
	bioassays.validateBioAssay = function()
	{
		var frm = document.forms['bioAssay'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		return true;
	}
	
	bioassays.save = function()
	{
		var frm = document.forms['bioAssay'];
		if (TabControl.validateActiveTab('settings'))
		{
			Annotations.saveModifiedAnnotationsToForm(frm);
			frm.submit();
		}
	}


	return bioassays;
}();

Doc.onLoad(BioAssays.initPage);
Doc.addElementInitializer(BioAssays.initElements);

